/*
 * Copyright (c) 2002 Sosnoski Software Solutions, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * End element handler for <code>option-trade</code> element and subelements.
 *
 * @author Dennis M. Sosnoski
 * @version 1.0
 */

public class OptionTradeHandler implements EndElementHandler
{
	/** Name of element containing entire trade. */
	public String TRADE_ELEMENT_NAME = "option-trade";
	
	/** Name of element containing stock symbol. */
	protected String SYMBOL_ELEMENT_NAME = "symbol";
	
	/** Name of element containing timestamp. */
	protected String TIME_ELEMENT_NAME = "time";
	
	/** Name of element containing option type ("call" or "put") */
	protected String TYPE_ELEMENT_NAME = "option-type";
	
	/** Name of element containing strike price. */
	protected String STRIKE_ELEMENT_NAME = "strike-price";
	
	/** Name of element containing expiration month. */
	protected String EXPIRE_ELEMENT_NAME = "expiration-month";
	
	/** Name of element containing trade price. */
	protected String PRICE_ELEMENT_NAME = "trade-price";
	
	/** Name of element containing quantity  */
	protected String QUANTITY_ELEMENT_NAME = "quantity";
	
	/** Symbol of stock traded. */
	protected String m_stockSymbol;
	
	/** Timestamp on trade. */
	protected String m_tradeTime;
	
	/** Flag for call option. */
	protected boolean m_isCallOption;
	
	/** Price for option exercise. */
	protected int m_strikePrice;
	
	/** Option expiration month. */
	protected int m_expirationMonth;
	
	/** Price at which trade took place. */
	protected double m_tradePrice;
	
	/** Number of options traded. */
	protected int m_sharesTraded;
	
	/**
	 * Actual handler method. This is called for each element end, so we
	 * just need to handle the passed content appropriately.
	 *
	 * @param lname element local name
	 * @param content accumulated character data content for element, with
	 * leading and trailing whitespace trimmed
	 */
	 
	public void endElement(String lname, String content) {
		if (lname.equals(TRADE_ELEMENT_NAME)) {
			OptionTrack.recordTrade(m_stockSymbol, m_isCallOption, 
				m_expirationMonth, m_strikePrice, m_tradeTime, m_tradePrice,
				m_sharesTraded);
		} else {
			if (lname.equals(SYMBOL_ELEMENT_NAME)) {
				m_stockSymbol = content;
			} else if (lname.equals(TIME_ELEMENT_NAME)) {
				m_tradeTime = content;
			} else if (lname.equals(QUANTITY_ELEMENT_NAME)) {
				m_sharesTraded = Integer.parseInt(content);
			} else if (lname.equals(TYPE_ELEMENT_NAME)) {
				m_isCallOption = content.equals("call");
			} else if (lname.equals(STRIKE_ELEMENT_NAME)) {
				m_strikePrice = Integer.parseInt(content);
			} else if (lname.equals(PRICE_ELEMENT_NAME)) {
				m_tradePrice = Double.parseDouble(content);
			} else if (lname.equals(EXPIRE_ELEMENT_NAME)) {
				m_expirationMonth = Integer.parseInt(content);
			}
		}
	}
}
